setClass("VENNset", representation(setlist="list", intersectmatrix="matrix", complexitylevels="integer", vennlist="list"))

setAs(from="list", to="VENNset",
      def=function(from) {
        new("VENNset", setlist=from$setlist,
            intersectmatrix=from$intersectmatrix,
            complexitylevels=from$complexitylevels,
            vennlist=from$vennlist)
      })

#' overLapper
#'
#' @param setlist  list ofvectors
#' @description This is a custom function I got from the SystemPipeR package (https://github.com/tgirke/systemPipeR) but
#' since I only use it once I want to avoid loading it as a dependency as it is quite large. See the package for documentation.
#' Essentially it takes a list of vectors and gives you the identities of the overlaps.
#' @export
overLapper <- function(setlist, complexity="default", sep="_", cleanup=FALSE, keepdups=FALSE, type) {
  ## Default complexity levels
  if(complexity[1]=="default") complexity <- 1:length(setlist)

  ## Check validity of inputs
  if(class(setlist)!="list" | length(names(setlist))==0) stop("Unexpected input.
                                                              The input 'setlist' needs to be of class 'list' where each list component stores a
                                                              label set as 'vector' and the name of each label set is provided under the
                                                              name slot of each list component.")
  if(!all(sapply(setlist, is.vector) | sapply(setlist, is.null)) | length(setlist) < 2) stop("Unexpected input.
                                                                                             The input 'setlist' needs to be a list with at least 2 components each
                                                                                             containig a vector of set labels.")
  if(length(type)!=1 & all(c("vennsets", "intersects") %in% type)) stop("Argument 'type' needs to be assigned 'vennsets' or 'intersects'.")
  if(type=="vennsets" & !identical(complexity, 1:length(setlist))) stop("When assigning 'vennsets' to 'type', then 'complexity' needs to be assigned 'default' or '1:length(setlist)'.")

  ## Clean up of sample sets to minimize formatting issues
  if(cleanup==TRUE) {
    ## Set all characters to upper case
    setlist <- sapply(setlist, function(x) gsub("([A-Z])", "\\U\\1", x, perl=TRUE, ignore.case=TRUE))
    ## Remove leading and trailing spaces
    setlist <- sapply(setlist, function(x) gsub("^ {1,}| {1,}$", "", x, perl=TRUE, ignore.case=TRUE))
  }

  ## Append object counter to retain duplicates
  if(keepdups==TRUE) {
    dupCount <- function(setlist=setlist) {
      count <- table(setlist)
      paste(rep(names(count), count), unlist(sapply(count, function(x) seq(1, x))), sep=".")
    }
    mynames <- names(setlist)
    setlist <- lapply(setlist, function(x) dupCount(x)) # lapply necessary for numeric data!
    names(setlist) <- mynames
  }

  ## Create intersect matrix (removes duplicates!)
  setunion <- sort(unique(unlist(setlist)))
  setmatrix <- sapply(names(setlist), function(x) setunion %in% unique(setlist[[x]]))
  rownames(setmatrix) <- setunion
  storage.mode(setmatrix) <- "numeric"

  ## Create all possible sample combinations within requested complexity levels
  labels <- names(setlist)
  allcombl <- lapply(complexity, function(x) combn(labels, m=x, simplify=FALSE))
  allcombl <- unlist(allcombl, recursive=FALSE)
  complevels <- sapply(allcombl, length)

  ## Return intersect list for generated sample combinations
  if(type=="intersects") {
    OLlist <- sapply(seq(along=allcombl), function(x) setunion[rowSums(setmatrix[, rep(allcombl[[x]], 2)]) == 2 * length(allcombl[[x]])])
    names(OLlist) <- sapply(allcombl, paste, collapse=sep)
    OLlist <- list(setlist=setlist, intersectmatrix=setmatrix, complexitylevels=complevels, intersectlist=OLlist)
    intersectset <- as(OLlist, "INTERSECTset")
    return(intersectset)
  }

  ## Return Venn intersect list for generated sample combinations
  if(type=="vennsets") {
    vennSets <- function(setmatrix=setmatrix, allcombl=allcombl, index=1) {
      mycol1 <- which(colnames(setmatrix) %in% allcombl[[index]])
      mycol2 <- which(!colnames(setmatrix) %in% allcombl[[index]])
      cond1 <- rowSums(setmatrix[, rep(mycol1, 2)]) == 2 * length(mycol1)
      cond2 <- rowSums(setmatrix[, rep(mycol2, 2)]) == 0
      return(setunion[cond1 & cond2])
    }
    vennOLlist <- sapply(seq(along=allcombl), function(x) vennSets(setmatrix=setmatrix, allcombl=allcombl, index=x))
    names(vennOLlist) <- sapply(allcombl, paste, collapse=sep)
    OLlist <- list(setlist=setlist, intersectmatrix=setmatrix, complexitylevels=complevels, vennlist=vennOLlist)
    vennset <- as(OLlist, "VENNset")
    return(vennset)
  }
}
